!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #########
      SUBROUTINE FLAG_DIAG_UPDATE(OFRAC, ODIAG_GRID, K2M, OSURF_BUDGET, ORAD_BUDGET, OCOEF, &
                                  OSURF_VARS, KBEQ, KDSTEQ, ODIAG_OCEAN, OINTERPOL_SST,     &
                                  OWATER_PROFILE, OINTERPOL_TS, OSURF_EVAP_BUDGET, OFLOOD,  &
                                  OPGD_ISBA, OCH_NO_FLUX_ISBA, OSURF_MISC_BUDGET_ISBA,      &
                                  OPGD_TEB, OSURF_MISC_BUDGET_TEB  )
!     ############################################################
!
!!****  *FLAG_DIAG_UPDATE* - routine to modify selection of output fields
!!
!!    PURPOSE
!!    -------
!!
!!**  METHOD
!!    ------
!!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*	
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    02/2008 
!
!       B.Decharme  10/2009 flag to desactivate writing of pgd 
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
USE MODD_DIAG_SURF_ATM_n,ONLY : LFRAC, LDIAG_GRID, N2M, LSURF_BUDGET, LRAD_BUDGET, LCOEF, &
                                LSURF_VARS, LSURF_BUDGETC, LRESET_BUDGETC
!
USE MODD_SURF_ATM_n, ONLY : CSEA, CTOWN, CNATURE, CWATER
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*       0.1   Declarations of arguments
!              -------------------------
!
LOGICAL, INTENT(IN) :: OFRAC
LOGICAL, INTENT(IN) :: ODIAG_GRID
INTEGER, INTENT(IN) :: K2M
LOGICAL, INTENT(IN) :: OSURF_BUDGET
LOGICAL, INTENT(IN) :: ORAD_BUDGET
LOGICAL, INTENT(IN) :: OCOEF
LOGICAL, INTENT(IN) :: OSURF_VARS
!
INTEGER, INTENT(IN) :: KBEQ
INTEGER, INTENT(IN) :: KDSTEQ
!
LOGICAL, INTENT(IN) :: ODIAG_OCEAN
LOGICAL, INTENT(IN) :: OINTERPOL_SST
LOGICAL, INTENT(IN) :: OWATER_PROFILE
LOGICAL, INTENT(IN) :: OINTERPOL_TS
LOGICAL, INTENT(IN) :: OSURF_EVAP_BUDGET
LOGICAL, INTENT(IN) :: OFLOOD
LOGICAL, INTENT(IN) :: OSURF_MISC_BUDGET_ISBA
LOGICAL, INTENT(IN) :: OCH_NO_FLUX_ISBA
LOGICAL, INTENT(IN) :: OPGD_ISBA
LOGICAL, INTENT(IN) :: OSURF_MISC_BUDGET_TEB
LOGICAL, INTENT(IN) :: OPGD_TEB
!
!*       0.2   Declarations of local variables
!              -------------------------------
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE',0,ZHOOK_HANDLE)
!
LFRAC = OFRAC
LDIAG_GRID = ODIAG_GRID
N2M = K2M
LSURF_BUDGET = OSURF_BUDGET
LRAD_BUDGET = ORAD_BUDGET
LCOEF = OCOEF
LSURF_VARS = OSURF_VARS
!
IF (.NOT.LRESET_BUDGETC) LSURF_BUDGETC = .TRUE.
!
IF (CSEA=='SEAFLX') CALL FLAG_SEA_UPDATE(KBEQ, ODIAG_OCEAN, OINTERPOL_SST)
IF (CWATER=='FLAKE ' .OR. CWATER=='WATFLX') &
  CALL FLAG_WATER_UPDATE(KBEQ, OWATER_PROFILE, OINTERPOL_TS)
IF (CNATURE=='ISBA  ') CALL FLAG_ISBA_UPDATE(KBEQ, KDSTEQ, OSURF_EVAP_BUDGET, OFLOOD, &
                                OPGD_ISBA, OCH_NO_FLUX_ISBA, OSURF_MISC_BUDGET_ISBA)
IF (CTOWN=='TEB   ') CALL FLAG_TEB_UPDATE(KBEQ, OPGD_TEB, OSURF_MISC_BUDGET_TEB)
!
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE',1,ZHOOK_HANDLE)
!-------------------------------------------------------------------------------
CONTAINS
!
SUBROUTINE FLAG_SEA_UPDATE(KBEQ, ODIAG_OCEAN, OINTERPOL_SST)
!
USE MODD_CH_SEAFLUX_n, ONLY : NBEQ
USE MODD_DIAG_OCEAN_n,   ONLY : LDIAG_OCEAN
USE MODD_SEAFLUX_n,     ONLY : LINTERPOL_SST
!
IMPLICIT NONE
!
INTEGER, INTENT(IN) :: KBEQ
LOGICAL, INTENT(IN) :: ODIAG_OCEAN
LOGICAL, INTENT(IN) :: OINTERPOL_SST
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_SEA_UPDATE',0,ZHOOK_HANDLE)
NBEQ = KBEQ
LDIAG_OCEAN = ODIAG_OCEAN
LINTERPOL_SST = OINTERPOL_SST
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_SEA_UPDATE',1,ZHOOK_HANDLE)
!
END SUBROUTINE FLAG_SEA_UPDATE
!
SUBROUTINE FLAG_WATER_UPDATE(KBEQ, OWATER_PROFILE, OINTERPOL_TS)
!
USE MODD_CH_WATFLUX_n, ONLY : NBEQ
USE MODD_DIAG_MISC_FLAKE_n,ONLY : LWATER_PROFILE
USE MODD_WATFLUX_n,ONLY : LINTERPOL_TS
!
IMPLICIT NONE
!
INTEGER, INTENT(IN) :: KBEQ
LOGICAL, INTENT(IN) :: OWATER_PROFILE
LOGICAL, INTENT(IN) :: OINTERPOL_TS
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_WATER_UPDATE',0,ZHOOK_HANDLE)
NBEQ = KBEQ
IF (CWATER=='FLAKE ') LWATER_PROFILE = OWATER_PROFILE
IF (CWATER=='WATFLX') LINTERPOL_TS = OINTERPOL_TS
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_WATER_UPDATE',1,ZHOOK_HANDLE)
!
END SUBROUTINE FLAG_WATER_UPDATE
!
SUBROUTINE FLAG_ISBA_UPDATE(KBEQ, KDSTEQ, OSURF_EVAP_BUDGET, OFLOOD, &
                                OPGD, OCH_NO_FLUX, OSURF_MISC_BUDGET)
!
USE MODD_CH_ISBA_n, ONLY : NBEQ, NDSTEQ, LCH_NO_FLUX
USE MODD_DIAG_EVAP_ISBA_n, ONLY : LSURF_EVAP_BUDGET
USE MODD_DIAG_ISBA_n, ONLY : LPGD
USE MODD_ISBA_n, ONLY : LFLOOD
USE MODD_DIAG_MISC_ISBA_n, ONLY : LSURF_MISC_BUDGET
!
IMPLICIT NONE
!
INTEGER, INTENT(IN) :: KBEQ
INTEGER, INTENT(IN) :: KDSTEQ
LOGICAL, INTENT(IN) :: OSURF_EVAP_BUDGET
LOGICAL, INTENT(IN) :: OFLOOD
LOGICAL, INTENT(IN) :: OPGD
LOGICAL, INTENT(IN) :: OCH_NO_FLUX
LOGICAL, INTENT(IN) :: OSURF_MISC_BUDGET
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_ISBA_UPDATE',0,ZHOOK_HANDLE)
NBEQ = KBEQ
NDSTEQ = KDSTEQ
LSURF_EVAP_BUDGET = OSURF_EVAP_BUDGET
LFLOOD = OFLOOD
LPGD = OPGD
LCH_NO_FLUX = OCH_NO_FLUX
LSURF_MISC_BUDGET = OSURF_MISC_BUDGET
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_ISBA_UPDATE',1,ZHOOK_HANDLE)
!
END SUBROUTINE FLAG_ISBA_UPDATE
!
SUBROUTINE FLAG_TEB_UPDATE(KBEQ, OPGD, OSURF_MISC_BUDGET)
!
USE MODD_CH_TEB_n, ONLY : NBEQ
USE MODD_DIAG_TEB_n, ONLY : LPGD
USE MODD_DIAG_MISC_TEB_n, ONLY : LSURF_MISC_BUDGET
!
IMPLICIT NONE
!
INTEGER, INTENT(IN) :: KBEQ
LOGICAL, INTENT(IN) :: OPGD
LOGICAL, INTENT(IN) :: OSURF_MISC_BUDGET
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_TEB_UPDATE',0,ZHOOK_HANDLE)
NBEQ = KBEQ
LPGD = OPGD
LSURF_MISC_BUDGET = OSURF_MISC_BUDGET
IF (LHOOK) CALL DR_HOOK('FLAG_DIAG_UPDATE:FLAG_TEB_UPDATE',1,ZHOOK_HANDLE)
!
END SUBROUTINE FLAG_TEB_UPDATE
!
END SUBROUTINE FLAG_DIAG_UPDATE

