!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!######################
MODULE MODD_DIAG_TEB_GREENROOF_n
!######################
!
!!****  *MODD_DIAG_TEB_GREENROOF - declaration of diagnostics for ISBA scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!    Based on modd_diag_teb_gardenn
!!
!!    AUTHOR
!!    ------
!!	C. de Munck & A. Lemonsu   *Meteo France
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/2011
!
!*       0.   DECLARATIONS
!             ------------
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_TEB_GREENROOF_t
!------------------------------------------------------------------------------
!
!* variables for one patch
!
  REAL, POINTER, DIMENSION(:)   :: XRI     ! Bulk-Richardson number           (-)
  REAL, POINTER, DIMENSION(:)   :: XCD     ! drag coefficient for wind        (W/s2)
  REAL, POINTER, DIMENSION(:)   :: XCH     ! drag coefficient for heat        (W/s)
  REAL, POINTER, DIMENSION(:)   :: XCE     ! drag coefficient for vapor       (W/s/K)
  REAL, POINTER, DIMENSION(:)   :: XRN     ! net radiation at surface         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH      ! sensible heat flux               (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX  ! net soil-vegetation flux         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XTS     ! surface temperature              (K)
  REAL, POINTER, DIMENSION(:)   :: XTSRAD  ! radiative surface temperature    (K)
  REAL, POINTER, DIMENSION(:)   :: XQS     ! humidity at surface              (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XLWD    ! downward long wave radiation     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLWU    ! upward long wave radiation       (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWD    ! downward short wave radiation    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWU    ! upward short wave radiation      (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWBD ! downward short wave radiation by spectral band   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWBU ! upward short wave radiation by spectral band (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XFMU    ! horizontal momentum flux zonal   (m2/s2)
  REAL, POINTER, DIMENSION(:)   :: XFMV    ! horizontal momentum flux meridian (m2/s2)  
  !    
  REAL, POINTER, DIMENSION(:)   :: XZ0_WITH_SNOW  ! roughness length for momentum
                                                  ! for vegetation and snow    (m)
  REAL, POINTER, DIMENSION(:)   :: XZ0H_WITH_SNOW ! roughness length for heat
                                                  ! for vegetation and snow    (m)
  REAL, POINTER, DIMENSION(:)   :: XZ0EFF         ! effective roughness length for heat
                                                  ! for vegetation and snow    (m)
!
  REAL, POINTER, DIMENSION(:)   :: XLEI          ! sublimation latent heat flux     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLEG          ! latent heat of evaporation over the ground   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLEGI         ! surface soil ice sublimation                 (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLEV          ! latent heat of evaporation over vegetation   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLES          ! latent heat of evaporation over the snow     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLER          ! evaporation from canopy water interception   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLETR         ! evapotranspiration of the vegetation         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XEVAP         ! evapotranspiration                           (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XDRAIN        ! soil drainage flux                           (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XRUNOFF       ! sub-grid and supersaturation runoff          (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XHORT         ! sub-grid Horton runoff from the SGH scheme   (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XRRVEG        !  precipitation intercepted by the vegetation (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XMELT         ! snow melt                                    (kg/m2/s)       
  REAL, POINTER, DIMENSION(:)   :: XDRIP         ! dripping from the vegetation reservoir       (kg/m2/s)  
!
!* pack diag
!
  REAL, POINTER, DIMENSION(:)   :: XCG           ! heat capacity of the ground
  REAL, POINTER, DIMENSION(:)   :: XC1           ! coefficients for the moisure
  REAL, POINTER, DIMENSION(:)   :: XC2           ! equation.
  REAL, POINTER, DIMENSION(:)   :: XWGEQ         ! equilibrium volumetric water content
  REAL, POINTER, DIMENSION(:)   :: XCT           ! area-averaged heat capacity
  REAL, POINTER, DIMENSION(:)   :: XRS           ! stomatal resistance                            (s/m)
  REAL, POINTER, DIMENSION(:)   :: XCDN          ! neutral drag coefficient                      (-)
  REAL, POINTER, DIMENSION(:)   :: XHU           ! area averaged surface humidity coefficient    (-)
  REAL, POINTER, DIMENSION(:)   :: XHUG          ! baresoil surface humidity coefficient         (-)
  REAL, POINTER, DIMENSION(:)   :: XRESTORE      ! surface energy budget restore term            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XUSTAR        ! friction velocity                             (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XIACAN        ! PAR in the canopy at different gauss level    (micmolphot/m2/s)
!
! for ISBA-ES:3-L
  REAL, POINTER, DIMENSION(:,:) :: XSNOWTEMP     ! snow temperature profile (ISBA-ES:3-L)        (K)
  REAL, POINTER, DIMENSION(:,:) :: XSNOWLIQ      ! snow liquid water profile (ISBA-ES:3-L)       (m)
  REAL, POINTER, DIMENSION(:,:) :: XSNOWDZ       ! snow layer thicknesses                        (m)
  REAL, POINTER, DIMENSION(:)   :: XSNOWHMASS    ! heat content change due to mass changes in snowpack (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XMELTADV      ! advective energy from snow melt water!
!
!* budget summation variables for one patch
!
!
  REAL, POINTER, DIMENSION(:)   :: XHV           ! Halstead coefficient
!      
  REAL, POINTER, DIMENSION(:,:) :: XSWI        ! Soil wetness index
  REAL, POINTER, DIMENSION(:,:) :: XTSWI       ! Total soil wetness index
!     
  REAL, POINTER, DIMENSION(:)   :: XTWSNOW       ! Total snow reservoir
  REAL, POINTER, DIMENSION(:)   :: XTDSNOW       ! Total snow height
!
  REAL, POINTER, DIMENSION(:)   :: XALBT             ! Total Albedo
  REAL, POINTER, DIMENSION(:)   :: XEMIST            ! averaged emissivity                     (-)
!
  REAL, POINTER, DIMENSION(:)   :: XSEUIL        ! Irrigation threshold
!
  REAL, POINTER, DIMENSION(:)   :: XGPP          ! Gross Primary Production
  REAL, POINTER, DIMENSION(:)   :: XRESP_AUTO    ! Autotrophic respiration
  REAL, POINTER, DIMENSION(:)   :: XRESP_ECO     ! Ecosystem respiration
!
!------------------------------------------------------------------------------
!

END TYPE DIAG_TEB_GREENROOF_t

TYPE(DIAG_TEB_GREENROOF_t), ALLOCATABLE, TARGET, SAVE :: DIAG_TEB_GREENROOF_MODEL(:)

REAL, POINTER, DIMENSION(:)   :: XRI=>NULL()
!$OMP THREADPRIVATE(XRI)
REAL, POINTER, DIMENSION(:)   :: XCD=>NULL()
!$OMP THREADPRIVATE(XCD)
REAL, POINTER, DIMENSION(:)   :: XCH=>NULL()
!$OMP THREADPRIVATE(XCH)
REAL, POINTER, DIMENSION(:)   :: XCE=>NULL()
!$OMP THREADPRIVATE(XCE)
REAL, POINTER, DIMENSION(:)   :: XRN=>NULL()
!$OMP THREADPRIVATE(XRN)
REAL, POINTER, DIMENSION(:)   :: XH=>NULL()
!$OMP THREADPRIVATE(XH)
REAL, POINTER, DIMENSION(:)   :: XGFLUX=>NULL()
!$OMP THREADPRIVATE(XGFLUX)
REAL, POINTER, DIMENSION(:)   :: XTS=>NULL()
!$OMP THREADPRIVATE(XTS)
REAL, POINTER, DIMENSION(:)   :: XTSRAD=>NULL()
!$OMP THREADPRIVATE(XTSRAD)
REAL, POINTER, DIMENSION(:)   :: XQS=>NULL()
!$OMP THREADPRIVATE(XQS)
REAL, POINTER, DIMENSION(:)   :: XLWD=>NULL()
!$OMP THREADPRIVATE(XLWD)
REAL, POINTER, DIMENSION(:)   :: XLWU=>NULL()
!$OMP THREADPRIVATE(XLWU)
REAL, POINTER, DIMENSION(:)   :: XSWD=>NULL()
!$OMP THREADPRIVATE(XSWD)
REAL, POINTER, DIMENSION(:)   :: XSWU=>NULL()
!$OMP THREADPRIVATE(XSWU)
REAL, POINTER, DIMENSION(:,:) :: XSWBD=>NULL()
!$OMP THREADPRIVATE(XSWBD)
REAL, POINTER, DIMENSION(:,:) :: XSWBU=>NULL()
!$OMP THREADPRIVATE(XSWBU)
REAL, POINTER, DIMENSION(:)   :: XFMU=>NULL()
!$OMP THREADPRIVATE(XFMU)
REAL, POINTER, DIMENSION(:)   :: XFMV=>NULL()
!$OMP THREADPRIVATE(XFMV)

REAL, POINTER, DIMENSION(:)   :: XZ0_WITH_SNOW=>NULL()
!$OMP THREADPRIVATE(XZ0_WITH_SNOW)
REAL, POINTER, DIMENSION(:)   :: XZ0H_WITH_SNOW=>NULL()
!$OMP THREADPRIVATE(XZ0H_WITH_SNOW)
REAL, POINTER, DIMENSION(:)   :: XZ0EFF=>NULL()
!$OMP THREADPRIVATE(XZ0EFF)
!
REAL, POINTER, DIMENSION(:)   :: XLEI=>NULL()
!$OMP THREADPRIVATE(XLEI)
REAL, POINTER, DIMENSION(:)   :: XLEG=>NULL()
!$OMP THREADPRIVATE(XLEG)
REAL, POINTER, DIMENSION(:)   :: XLEGI=>NULL()
!$OMP THREADPRIVATE(XLEGI)
REAL, POINTER, DIMENSION(:)   :: XLEV=>NULL()
!$OMP THREADPRIVATE(XLEV)
REAL, POINTER, DIMENSION(:)   :: XLES=>NULL()
!$OMP THREADPRIVATE(XLES)
REAL, POINTER, DIMENSION(:)   :: XLER=>NULL()
!$OMP THREADPRIVATE(XLER)
REAL, POINTER, DIMENSION(:)   :: XLETR=>NULL()
!$OMP THREADPRIVATE(XLETR)
REAL, POINTER, DIMENSION(:)   :: XEVAP=>NULL()
!$OMP THREADPRIVATE(XEVAP)
REAL, POINTER, DIMENSION(:)   :: XDRAIN=>NULL()
!$OMP THREADPRIVATE(XDRAIN)
REAL, POINTER, DIMENSION(:)   :: XRUNOFF=>NULL()
!$OMP THREADPRIVATE(XRUNOFF)
REAL, POINTER, DIMENSION(:)   :: XHORT=>NULL()
!$OMP THREADPRIVATE(XHORT)
REAL, POINTER, DIMENSION(:)   :: XDRIP=>NULL()
!$OMP THREADPRIVATE(XDRIP)
REAL, POINTER, DIMENSION(:)   :: XMELT=>NULL()
!$OMP THREADPRIVATE(XMELT)
REAL, POINTER, DIMENSION(:)   :: XRRVEG=>NULL()
!$OMP THREADPRIVATE(XRRVEG)
REAL, POINTER, DIMENSION(:)   :: XCG=>NULL()
!$OMP THREADPRIVATE(XCG)
REAL, POINTER, DIMENSION(:)   :: XC1=>NULL()
!$OMP THREADPRIVATE(XC1)
REAL, POINTER, DIMENSION(:)   :: XC2=>NULL()
!$OMP THREADPRIVATE(XC2)
REAL, POINTER, DIMENSION(:)   :: XWGEQ=>NULL()
!$OMP THREADPRIVATE(XWGEQ)
REAL, POINTER, DIMENSION(:)   :: XCT=>NULL()
!$OMP THREADPRIVATE(XCT)
REAL, POINTER, DIMENSION(:)   :: XRS=>NULL()
!$OMP THREADPRIVATE(XRS)
REAL, POINTER, DIMENSION(:)   :: XCDN=>NULL()
!$OMP THREADPRIVATE(XCDN)
REAL, POINTER, DIMENSION(:)   :: XHU=>NULL()
!$OMP THREADPRIVATE(XHU)
REAL, POINTER, DIMENSION(:)   :: XHUG=>NULL()
!$OMP THREADPRIVATE(XHUG)
REAL, POINTER, DIMENSION(:)   :: XRESTORE=>NULL()
!$OMP THREADPRIVATE(XRESTORE)
REAL, POINTER, DIMENSION(:)   :: XUSTAR=>NULL()
!$OMP THREADPRIVATE(XUSTAR)
REAL, POINTER, DIMENSION(:,:) :: XIACAN=>NULL()
!$OMP THREADPRIVATE(XIACAN)
REAL, POINTER, DIMENSION(:,:) :: XSNOWTEMP=>NULL()
!$OMP THREADPRIVATE(XSNOWTEMP)
REAL, POINTER, DIMENSION(:,:) :: XSNOWLIQ=>NULL()
!$OMP THREADPRIVATE(XSNOWLIQ)
REAL, POINTER, DIMENSION(:,:) :: XSNOWDZ=>NULL()
!$OMP THREADPRIVATE(XSNOWDZ)
REAL, POINTER, DIMENSION(:)   :: XSNOWHMASS=>NULL()
!$OMP THREADPRIVATE(XSNOWHMASS)
REAL, POINTER, DIMENSION(:)   :: XMELTADV=>NULL()
!$OMP THREADPRIVATE(XMELTADV)

!
REAL, POINTER, DIMENSION(:)   :: XHV=>NULL()
!$OMP THREADPRIVATE(XHV)
REAL, POINTER, DIMENSION(:)   :: XEMIST=>NULL()
!$OMP THREADPRIVATE(XEMIST)
REAL, POINTER, DIMENSION(:)   :: XALBT=>NULL()
!$OMP THREADPRIVATE(XALBT)
REAL, POINTER, DIMENSION(:)   :: XSEUIL=>NULL()
!$OMP THREADPRIVATE(XSEUIL)
REAL, POINTER, DIMENSION(:)   :: XGPP=>NULL()
!$OMP THREADPRIVATE(XGPP)
REAL, POINTER, DIMENSION(:)   :: XRESP_AUTO=>NULL()
!$OMP THREADPRIVATE(XRESP_AUTO)
REAL, POINTER, DIMENSION(:)   :: XRESP_ECO=>NULL()
!$OMP THREADPRIVATE(XRESP_ECO)
!
REAL, POINTER, DIMENSION(:,:) :: XSWI=>NULL()
!$OMP THREADPRIVATE(XSWI)
REAL, POINTER, DIMENSION(:,:) :: XTSWI=>NULL()
!$OMP THREADPRIVATE(XTSWI)
!
REAL, POINTER, DIMENSION(:)   :: XTWSNOW=>NULL()
!$OMP THREADPRIVATE(XTWSNOW)
REAL, POINTER, DIMENSION(:)   :: XTDSNOW=>NULL()
!$OMP THREADPRIVATE(XTDSNOW)
!

CONTAINS

SUBROUTINE DIAG_TEB_GREENROOF_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRI=>XRI
DIAG_TEB_GREENROOF_MODEL(KFROM)%XCD=>XCD
DIAG_TEB_GREENROOF_MODEL(KFROM)%XCH=>XCH
DIAG_TEB_GREENROOF_MODEL(KFROM)%XCE=>XCE
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRN=>XRN
DIAG_TEB_GREENROOF_MODEL(KFROM)%XH=>XH
DIAG_TEB_GREENROOF_MODEL(KFROM)%XGFLUX=>XGFLUX
DIAG_TEB_GREENROOF_MODEL(KFROM)%XTS=>XTS
DIAG_TEB_GREENROOF_MODEL(KFROM)%XTSRAD=>XTSRAD
DIAG_TEB_GREENROOF_MODEL(KFROM)%XQS=>XQS
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLWD=>XLWD
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLWU=>XLWU
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSWD=>XSWD
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSWU=>XSWU
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSWBD=>XSWBD
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSWBU=>XSWBU
DIAG_TEB_GREENROOF_MODEL(KFROM)%XFMU=>XFMU
DIAG_TEB_GREENROOF_MODEL(KFROM)%XFMV=>XFMV
DIAG_TEB_GREENROOF_MODEL(KFROM)%XZ0_WITH_SNOW=>XZ0_WITH_SNOW
DIAG_TEB_GREENROOF_MODEL(KFROM)%XZ0H_WITH_SNOW=>XZ0H_WITH_SNOW
DIAG_TEB_GREENROOF_MODEL(KFROM)%XZ0EFF=>XZ0EFF
!
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLEI=>XLEI
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLEG=>XLEG
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLEGI=>XLEGI
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLEV=>XLEV
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLES=>XLES
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLER=>XLER
DIAG_TEB_GREENROOF_MODEL(KFROM)%XLETR=>XLETR
DIAG_TEB_GREENROOF_MODEL(KFROM)%XEVAP=>XEVAP
DIAG_TEB_GREENROOF_MODEL(KFROM)%XDRAIN=>XDRAIN
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRUNOFF=>XRUNOFF
DIAG_TEB_GREENROOF_MODEL(KFROM)%XHORT=>XHORT
DIAG_TEB_GREENROOF_MODEL(KFROM)%XDRIP=>XDRIP
DIAG_TEB_GREENROOF_MODEL(KFROM)%XMELT=>XMELT
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRRVEG=>XRRVEG
DIAG_TEB_GREENROOF_MODEL(KFROM)%XCG=>XCG
DIAG_TEB_GREENROOF_MODEL(KFROM)%XC1=>XC1
DIAG_TEB_GREENROOF_MODEL(KFROM)%XC2=>XC2
DIAG_TEB_GREENROOF_MODEL(KFROM)%XWGEQ=>XWGEQ
DIAG_TEB_GREENROOF_MODEL(KFROM)%XCT=>XCT
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRS=>XRS
DIAG_TEB_GREENROOF_MODEL(KFROM)%XCDN=>XCDN
DIAG_TEB_GREENROOF_MODEL(KFROM)%XHU=>XHU
DIAG_TEB_GREENROOF_MODEL(KFROM)%XHUG=>XHUG
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRESTORE=>XRESTORE
DIAG_TEB_GREENROOF_MODEL(KFROM)%XUSTAR=>XUSTAR
DIAG_TEB_GREENROOF_MODEL(KFROM)%XIACAN=>XIACAN
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSNOWTEMP=>XSNOWTEMP
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSNOWLIQ=>XSNOWLIQ
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSNOWDZ=>XSNOWDZ
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSNOWHMASS=>XSNOWHMASS
DIAG_TEB_GREENROOF_MODEL(KFROM)%XMELTADV=>XMELTADV
!
DIAG_TEB_GREENROOF_MODEL(KFROM)%XHV=>XHV
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSWI=>XSWI
DIAG_TEB_GREENROOF_MODEL(KFROM)%XTSWI=>XTSWI
DIAG_TEB_GREENROOF_MODEL(KFROM)%XTWSNOW=>XTWSNOW
DIAG_TEB_GREENROOF_MODEL(KFROM)%XTDSNOW=>XTDSNOW
DIAG_TEB_GREENROOF_MODEL(KFROM)%XEMIST=>XEMIST
DIAG_TEB_GREENROOF_MODEL(KFROM)%XALBT=>XALBT
DIAG_TEB_GREENROOF_MODEL(KFROM)%XSEUIL=>XSEUIL
DIAG_TEB_GREENROOF_MODEL(KFROM)%XGPP=>XGPP
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRESP_AUTO=>XRESP_AUTO
DIAG_TEB_GREENROOF_MODEL(KFROM)%XRESP_ECO=>XRESP_ECO
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_TEB_GREENROOF_N:DIAG_TEB_GREENROOF_GOTO_MODEL',0,ZHOOK_HANDLE)
XRI=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRI
XCD=>DIAG_TEB_GREENROOF_MODEL(KTO)%XCD
XCH=>DIAG_TEB_GREENROOF_MODEL(KTO)%XCH
XCE=>DIAG_TEB_GREENROOF_MODEL(KTO)%XCE
XRN=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRN
XH=>DIAG_TEB_GREENROOF_MODEL(KTO)%XH
XGFLUX=>DIAG_TEB_GREENROOF_MODEL(KTO)%XGFLUX
XTS=>DIAG_TEB_GREENROOF_MODEL(KTO)%XTS
XTSRAD=>DIAG_TEB_GREENROOF_MODEL(KTO)%XTSRAD
XQS=>DIAG_TEB_GREENROOF_MODEL(KTO)%XQS
XLWD=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLWD
XLWU=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLWU
XSWD=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSWD
XSWU=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSWU
XSWBD=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSWBD
XSWBU=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSWBU
XFMU=>DIAG_TEB_GREENROOF_MODEL(KTO)%XFMU
XFMV=>DIAG_TEB_GREENROOF_MODEL(KTO)%XFMV
XZ0_WITH_SNOW=>DIAG_TEB_GREENROOF_MODEL(KTO)%XZ0_WITH_SNOW
XZ0H_WITH_SNOW=>DIAG_TEB_GREENROOF_MODEL(KTO)%XZ0H_WITH_SNOW
XZ0EFF=>DIAG_TEB_GREENROOF_MODEL(KTO)%XZ0EFF
!
XLEI=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLEI
XLEG=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLEG
XLEGI=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLEGI
XLEV=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLEV
XLES=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLES
XLER=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLER
XLETR=>DIAG_TEB_GREENROOF_MODEL(KTO)%XLETR
XEVAP=>DIAG_TEB_GREENROOF_MODEL(KTO)%XEVAP
XDRAIN=>DIAG_TEB_GREENROOF_MODEL(KTO)%XDRAIN
XRUNOFF=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRUNOFF
XHORT=>DIAG_TEB_GREENROOF_MODEL(KTO)%XHORT
XDRIP=>DIAG_TEB_GREENROOF_MODEL(KTO)%XDRIP
XMELT=>DIAG_TEB_GREENROOF_MODEL(KTO)%XMELT
XRRVEG=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRRVEG
XCG=>DIAG_TEB_GREENROOF_MODEL(KTO)%XCG
XC1=>DIAG_TEB_GREENROOF_MODEL(KTO)%XC1
XC2=>DIAG_TEB_GREENROOF_MODEL(KTO)%XC2
XWGEQ=>DIAG_TEB_GREENROOF_MODEL(KTO)%XWGEQ
XCT=>DIAG_TEB_GREENROOF_MODEL(KTO)%XCT
XRS=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRS
XCDN=>DIAG_TEB_GREENROOF_MODEL(KTO)%XCDN
XHU=>DIAG_TEB_GREENROOF_MODEL(KTO)%XHU
XHUG=>DIAG_TEB_GREENROOF_MODEL(KTO)%XHUG
XRESTORE=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRESTORE
XUSTAR=>DIAG_TEB_GREENROOF_MODEL(KTO)%XUSTAR
XIACAN=>DIAG_TEB_GREENROOF_MODEL(KTO)%XIACAN
XSNOWTEMP=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSNOWTEMP
XSNOWLIQ=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSNOWLIQ
XSNOWDZ=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSNOWDZ
XSNOWHMASS=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSNOWHMASS
XMELTADV=>DIAG_TEB_GREENROOF_MODEL(KTO)%XMELTADV
!
XHV=>DIAG_TEB_GREENROOF_MODEL(KTO)%XHV
XSWI=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSWI
XTSWI=>DIAG_TEB_GREENROOF_MODEL(KTO)%XTSWI
XTWSNOW=>DIAG_TEB_GREENROOF_MODEL(KTO)%XTWSNOW
XTDSNOW=>DIAG_TEB_GREENROOF_MODEL(KTO)%XTDSNOW
XEMIST=>DIAG_TEB_GREENROOF_MODEL(KTO)%XEMIST
XALBT=>DIAG_TEB_GREENROOF_MODEL(KTO)%XALBT
XSEUIL=>DIAG_TEB_GREENROOF_MODEL(KTO)%XSEUIL
XGPP=>DIAG_TEB_GREENROOF_MODEL(KTO)%XGPP
XRESP_AUTO=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRESP_AUTO
XRESP_ECO=>DIAG_TEB_GREENROOF_MODEL(KTO)%XRESP_ECO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_TEB_GREENROOF_N:DIAG_TEB_GREENROOF_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_TEB_GREENROOF_GOTO_MODEL

SUBROUTINE DIAG_TEB_GREENROOF_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_TEB_GREENROOF_N:DIAG_TEB_GREENROOF_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_TEB_GREENROOF_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRI)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XCD)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XCH)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XCE)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRN)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XH)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XGFLUX)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XTS)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XTSRAD)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XQS)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLWD)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLWU)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSWD)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSWU)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSWBD)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSWBU)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XFMU)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XFMV)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XZ0_WITH_SNOW)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XZ0H_WITH_SNOW)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XZ0EFF)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLEI)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLEG)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLEGI)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLEV)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLES)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLER)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XLETR)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XEVAP)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XDRAIN)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRUNOFF)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XHORT)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRRVEG)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XMELT)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XDRIP)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XCG)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XC1)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XC2)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XWGEQ)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XCT)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRS)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XCDN)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XHU)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XHUG)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRESTORE)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XUSTAR)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XIACAN)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSNOWTEMP)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSNOWLIQ)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSNOWDZ)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSNOWHMASS)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XMELTADV)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XHV)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSWI)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XTSWI)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XTWSNOW)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XTDSNOW)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XALBT)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XEMIST)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XSEUIL)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XGPP)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRESP_AUTO)
  NULLIFY(DIAG_TEB_GREENROOF_MODEL(J)%XRESP_ECO)
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_TEB_GREENROOF_N:DIAG_TEB_GREENROOF_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_TEB_GREENROOF_ALLOC

SUBROUTINE DIAG_TEB_GREENROOF_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_TEB_GREENROOF_N:DIAG_TEB_GREENROOF_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_TEB_GREENROOF_MODEL)) DEALLOCATE(DIAG_TEB_GREENROOF_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_TEB_GREENROOF_N:DIAG_TEB_GREENROOF_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_TEB_GREENROOF_DEALLO

END MODULE MODD_DIAG_TEB_GREENROOF_n
