!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #####################
      MODULE MODD_GR_BIOG_GREENROOF_n
!     ######################
!
!!
!!!!****  *MODD_GR_BIOG_GREENROOF_n* - Declaration of variables for biogenic emissions 
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!    AUTHOR
!!    ------
!!      F. Solmon  *LA*
!!
!!    MODIFICATIONS
!!    -------------
!!    P. Tulet  30/07/03 externalisation of biogenics fluxes
!!     
!*       0.   DECLARATIONS
!             ----------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE GR_BIOG_GREENROOF_t
!
!
!
!* Emission potential for isoprene and monoterpenes 
  REAL, DIMENSION(:), POINTER :: XISOPOT 
  REAL, DIMENSION(:), POINTER :: XMONOPOT 
!
!* Radiation at different level(cf Gauss) in the canopy
  REAL, DIMENSION(:), POINTER :: XP_IACAN !pack radiation
  REAL, DIMENSION(:,:,:),POINTER ::XIACAN ! PAR at 3 gauss level for each patch
!
!* XFISO  = isoprene emission flux (ppp.m.s-1)
!  XFMONO = monoterpenes emission flux (ppp m s-1)
  REAL, DIMENSION(:), POINTER :: XFISO, XFMONO
!
!
END TYPE GR_BIOG_GREENROOF_t

TYPE(GR_BIOG_GREENROOF_t), ALLOCATABLE, TARGET, SAVE :: GR_BIOG_GREENROOF_MODEL(:)

REAL, DIMENSION(:), POINTER :: XISOPOT=>NULL()
!$OMP THREADPRIVATE(XISOPOT)
REAL, DIMENSION(:), POINTER :: XMONOPOT=>NULL()
!$OMP THREADPRIVATE(XMONOPOT)
REAL, DIMENSION(:), POINTER :: XP_IACAN=>NULL()
!$OMP THREADPRIVATE(XP_IACAN)
REAL, DIMENSION(:,:,:),POINTER :: XIACAN=>NULL()
!$OMP THREADPRIVATE(XIACAN)
REAL, DIMENSION(:), POINTER :: XFISO=>NULL(), XFMONO=>NULL()
!$OMP THREADPRIVATE(XFISO, XFMONO)

CONTAINS

SUBROUTINE GR_BIOG_GREENROOF_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
GR_BIOG_GREENROOF_MODEL(KFROM)%XISOPOT=>XISOPOT
GR_BIOG_GREENROOF_MODEL(KFROM)%XMONOPOT=>XMONOPOT
GR_BIOG_GREENROOF_MODEL(KFROM)%XP_IACAN=>XP_IACAN
GR_BIOG_GREENROOF_MODEL(KFROM)%XIACAN=>XIACAN
GR_BIOG_GREENROOF_MODEL(KFROM)%XFISO=>XFISO
GR_BIOG_GREENROOF_MODEL(KFROM)%XFMONO=>XFMONO
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_GR_BIOG_GREENROOF_N:GR_BIOG_GREENROOF_GOTO_MODEL',0,ZHOOK_HANDLE)
XISOPOT=>GR_BIOG_GREENROOF_MODEL(KTO)%XISOPOT
XMONOPOT=>GR_BIOG_GREENROOF_MODEL(KTO)%XMONOPOT
XP_IACAN=>GR_BIOG_GREENROOF_MODEL(KTO)%XP_IACAN
XIACAN=>GR_BIOG_GREENROOF_MODEL(KTO)%XIACAN
XFISO=>GR_BIOG_GREENROOF_MODEL(KTO)%XFISO
XFMONO=>GR_BIOG_GREENROOF_MODEL(KTO)%XFMONO
IF (LHOOK) CALL DR_HOOK('MODD_GR_BIOG_GREENROOF_N:GR_BIOG_GREENROOF_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE GR_BIOG_GREENROOF_GOTO_MODEL

SUBROUTINE GR_BIOG_GREENROOF_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_GR_BIOG_GREENROOF_N:GR_BIOG_GREENROOF_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(GR_BIOG_GREENROOF_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(GR_BIOG_GREENROOF_MODEL(J)%XISOPOT)
  NULLIFY(GR_BIOG_GREENROOF_MODEL(J)%XMONOPOT)
  NULLIFY(GR_BIOG_GREENROOF_MODEL(J)%XP_IACAN)
  NULLIFY(GR_BIOG_GREENROOF_MODEL(J)%XIACAN)
  NULLIFY(GR_BIOG_GREENROOF_MODEL(J)%XFISO)
  NULLIFY(GR_BIOG_GREENROOF_MODEL(J)%XFMONO)
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_GR_BIOG_GREENROOF_N:GR_BIOG_GREENROOF_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE GR_BIOG_GREENROOF_ALLOC

SUBROUTINE GR_BIOG_GREENROOF_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_GR_BIOG_GREENROOF_N:GR_BIOG_GREENROOF_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(GR_BIOG_GREENROOF_MODEL)) DEALLOCATE(GR_BIOG_GREENROOF_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_GR_BIOG_GREENROOF_N:GR_BIOG_GREENROOF_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE GR_BIOG_GREENROOF_DEALLO

END MODULE MODD_GR_BIOG_GREENROOF_n
